/*
 * JDU: Disk Usage
 * 
 * Copyright (c) 2001-2005 Jason Pratt
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

package org.jasonpratt.jdu;

import java.awt.*;
import javax.swing.*;
import java.awt.event.*;
import javax.swing.border.*;

/**
 * @author Jason Pratt
 */
public class JDUFrame extends javax.swing.JFrame {
	private static final long serialVersionUID = 4415414101347616324L;

	public final static String version = "1.1";

	protected AboutDialog aboutDialog = new AboutDialog( this );
	protected GettingStartedDialog gettingStartedDialog = new GettingStartedDialog( this );
	protected TableView tableView = new TableView();
	protected java.awt.Color messageColor = java.awt.Color.black;
	protected java.awt.Color errorColor = new java.awt.Color( 150, 20, 20 );
	protected javax.swing.JFileChooser fileChooser = new javax.swing.JFileChooser();

	////////////////////////
	// anonymous classes
	////////////////////////

	protected ProgressObserver progressObserver =  new ProgressObserver() {
		private long lastTime;

		public void updateCurrentItem( String itemName ) {
			showMessage( "Working: " + itemName );
			lastTime = System.currentTimeMillis();
			try {
				Thread.sleep( 1 );
			} catch( InterruptedException e ) {
				e.printStackTrace();
			}
		}

		public boolean needsUpdate() {
			return (System.currentTimeMillis() - lastTime) > 50;
		}
	};

	protected org.jasonpratt.jdu.event.TableViewListener tableViewListener = new org.jasonpratt.jdu.event.TableViewListener() {
		public void rootChanged( Directory oldRoot, Directory newRoot ) {
			if( newRoot != null ) {
				showMessage( newRoot.getFile().getAbsolutePath() );
			} else {
				showMessage( "" );
			}
		}
	};

	/////////////////
	// Constructor
	/////////////////

	public JDUFrame() {
		jbInit();
		guiInit();
	}

	private void guiInit() {
		mainPanel.add( tableView, java.awt.BorderLayout.CENTER );
		depthComboBox.addItem( "2" );
		depthComboBox.addItem( "3" );
		depthComboBox.addItem( "4" );
		depthComboBox.addItem( "5" );
		depthComboBox.addItem( "6" );
		depthComboBox.addItem( "7" );
		depthComboBox.addItem( "8" );
		depthComboBox.setSelectedItem( "4" );

		tableView.addTableViewListener( tableViewListener );

		fileChooser.setDialogTitle( "Choose a directory..." );
		fileChooser.setApproveButtonText( "Set Start Directory" );
		fileChooser.setDialogType( javax.swing.JFileChooser.OPEN_DIALOG );
		fileChooser.setFileSelectionMode( javax.swing.JFileChooser.DIRECTORIES_ONLY );
		fileChooser.setApproveButtonToolTipText( "" );

		java.awt.Dimension screenSize = java.awt.Toolkit.getDefaultToolkit().getScreenSize();
		int width = Math.max( screenSize.width/2, 610 );
		int height = (int)(((Math.sqrt( 5.0 ) - 1.0)/2.0)*width);
		setSize( width, height );
		centerComponentOnScreen( this );

		javax.swing.ButtonGroup lookAndFeelGroup = new javax.swing.ButtonGroup();
		lookAndFeelGroup.add( metalLAFItem );
		lookAndFeelGroup.add( platformLAFItem );
		String platformLAFName = UIManager.getSystemLookAndFeelClassName();
		platformLAFName = platformLAFName.substring( platformLAFName.lastIndexOf( '.' ) + 1, platformLAFName.lastIndexOf( "LookAndFeel" ) );
		platformLAFItem.setText( platformLAFName );

		centerComponentOnScreen( aboutDialog );

		setIconImage( java.awt.Toolkit.getDefaultToolkit().getImage( JDUFrame.class.getResource( "images/jduLogo.gif" ) ) );
	}

	/////////////
	// utility
	/////////////

	public static void centerComponentOnScreen( java.awt.Component c ) {
		java.awt.Dimension screenSize = java.awt.Toolkit.getDefaultToolkit().getScreenSize();
		int x = (screenSize.width - c.getWidth())/2;
		int y = (screenSize.height - c.getHeight())/2;
		c.setLocation( x, y );
	}

	public static void openURL( String urlString ) throws java.io.IOException {
		if( (System.getProperty( "os.name" ) != null) && System.getProperty( "os.name" ).startsWith( "Windows" ) ) {
			Runtime.getRuntime().exec( "rundll32 url.dll,FileProtocolHandler " + urlString );
		} else {
			// try netscape
			String[] cmd = new String[] { "netscape", urlString };
			Runtime.getRuntime().exec( cmd );
		}
	}

	public void updateView() {
		final java.io.File file = new java.io.File( rootTextField.getText() );
		if( file.exists() ) {
			if( file.isDirectory() ) {
				SwingWorker worker = new SwingWorker() {
					public Object construct() {
						rootTextField.setEnabled( false );
						depthComboBox.setEnabled( false );
						goButton.setEnabled( false );
						browseButton.setEnabled( false );
						tableView.setRoot( new Directory( null, file, progressObserver ) );
						rootTextField.setEnabled( true );
						depthComboBox.setEnabled( true );
						goButton.setEnabled( true );
						browseButton.setEnabled( true );
						showMessage( "Done" );
						return null;
					}
				};
				worker.start();
			} else {
				showError( "Not a directory: " + file.getAbsolutePath() );
			}
		} else {
			showError( "Directory does not exist: " + file.getAbsolutePath() );
		}
	}

	public void showMessage( String message ) {
		statusLabel.setForeground( messageColor );
		statusLabel.setText( message );
	}

	public void showError( String message ) {
		statusLabel.setForeground( errorColor );
		statusLabel.setText( message );
	}

	///////////////
	// Callbacks
	///////////////

	void rootTextField_actionPerformed(ActionEvent e) {
		updateView();
	}

	void depthComboBox_actionPerformed(ActionEvent e) {
		tableView.setDepth( Integer.parseInt( (String)depthComboBox.getSelectedItem() ) );
	}

	void goButton_actionPerformed(ActionEvent e) {
		updateView();
	}

	void browseButton_actionPerformed(ActionEvent e) {
		int result = fileChooser.showDialog( this, null );
		if( result == javax.swing.JFileChooser.APPROVE_OPTION ) {
			java.io.File file = fileChooser.getSelectedFile();
			rootTextField.setText( file.getAbsolutePath() );
			updateView();
		} else if( result == javax.swing.JFileChooser.ERROR_OPTION ) {
			showError( "Error while choosing directory." );
		} else if( result == javax.swing.JFileChooser.CANCEL_OPTION ) {
			// do nothing
		}
	}

	void preferencesItem_actionPerformed(ActionEvent e) {

	}

	void gettingStartedItem_actionPerformed(ActionEvent e) {
		gettingStartedDialog.setVisible( true );
	}

	void aboutItem_actionPerformed(ActionEvent e) {
		aboutDialog.setVisible( true );
	}

	void metalLAFItem_actionPerformed(ActionEvent e) {
		try {
			javax.swing.UIManager.setLookAndFeel( "javax.swing.plaf.metal.MetalLookAndFeel" );
			SwingUtilities.updateComponentTreeUI( this );
		} catch( Throwable t ) {
			t.printStackTrace();
		}
	}

	void platformLAFItem_actionPerformed(ActionEvent e) {
		try {
			javax.swing.UIManager.setLookAndFeel( UIManager.getSystemLookAndFeelClassName() );
			SwingUtilities.updateComponentTreeUI( this );
		} catch( Throwable t ) {
			t.printStackTrace();
		}
	}

	//////////////////
	// Autogenerated
	//////////////////

	GridBagLayout gridBagLayout1 = new GridBagLayout();
	JLabel rootLabel = new JLabel();
	JTextField rootTextField = new JTextField();
	JPanel mainPanel = new JPanel();
	BorderLayout borderLayout1 = new BorderLayout();
	JLabel depthLabel = new JLabel();
	JComboBox depthComboBox = new JComboBox();
	JPanel statusBar = new JPanel();
	Border loweredBevelBorder;
	Border border1;
	Border border2;
	Border border3;
	GridBagLayout gridBagLayout2 = new GridBagLayout();
	JLabel statusLabel = new JLabel();
	JButton goButton = new JButton();
	JButton browseButton = new JButton();
	JMenuBar menuBar = new JMenuBar();
	JMenu editMenu = new JMenu();
	JMenu helpMenu = new JMenu();
	JMenuItem preferencesItem = new JMenuItem();
	JMenuItem gettingStartedItem = new JMenuItem();
	JMenuItem aboutItem = new JMenuItem();
	JMenu lookAndFeelMenu = new JMenu();
	JRadioButtonMenuItem metalLAFItem = new JRadioButtonMenuItem();
	JRadioButtonMenuItem platformLAFItem = new JRadioButtonMenuItem();

	private void jbInit() {
		loweredBevelBorder = BorderFactory.createBevelBorder( javax.swing.border.BevelBorder.LOWERED );
		border1 = BorderFactory.createEmptyBorder();
		border2 = BorderFactory.createEmptyBorder();
		border3 = BorderFactory.createCompoundBorder(BorderFactory.createEmptyBorder(),loweredBevelBorder);
		rootLabel.setText("Start Directory:");
		this.getContentPane().setLayout(gridBagLayout1);
		mainPanel.setLayout(borderLayout1);
		this.setDefaultCloseOperation(3);
		this.setJMenuBar(menuBar);
		this.setTitle("JDU: Disk Usage");
		rootTextField.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(ActionEvent e) {
				rootTextField_actionPerformed(e);
			}
		});
		depthLabel.setText("Depth:");
		depthComboBox.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(ActionEvent e) {
				depthComboBox_actionPerformed(e);
			}
		});
		statusBar.setBorder(border3);
		statusBar.setLayout(gridBagLayout2);
		mainPanel.setBorder(border1);
		statusLabel.setForeground( messageColor );
		statusLabel.setText("No start directory chosen");
		goButton.setMargin(new Insets(1, 8, 1, 8));
		goButton.setText("Go");
		goButton.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(ActionEvent e) {
				goButton_actionPerformed(e);
			}
		});
		browseButton.setMargin(new Insets(1, 8, 1, 8));
		browseButton.setText("Browse...");
		browseButton.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(ActionEvent e) {
				browseButton_actionPerformed(e);
			}
		});
		editMenu.setText("Edit");
		helpMenu.setText("Help");
		preferencesItem.setText("Preferences...");
		preferencesItem.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(ActionEvent e) {
				preferencesItem_actionPerformed(e);
			}
		});
		gettingStartedItem.setText("Getting Started...");
		gettingStartedItem.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(ActionEvent e) {
				gettingStartedItem_actionPerformed(e);
			}
		});
		aboutItem.setText("About...");
		aboutItem.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(ActionEvent e) {
				aboutItem_actionPerformed(e);
			}
		});
		lookAndFeelMenu.setText("Look and Feel");
		metalLAFItem.setText("Metal");
		metalLAFItem.setSelected(true);
		metalLAFItem.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(ActionEvent e) {
				metalLAFItem_actionPerformed(e);
			}
		});
		platformLAFItem.setText("");
		platformLAFItem.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(ActionEvent e) {
				platformLAFItem_actionPerformed(e);
			}
		});
		this.getContentPane().add(rootLabel,        new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0
			,GridBagConstraints.SOUTH, GridBagConstraints.NONE, new Insets(4, 4, 4, 4), 0, 0));
		this.getContentPane().add(rootTextField,          new GridBagConstraints(1, 0, 1, 1, 1.0, 0.0
			,GridBagConstraints.SOUTH, GridBagConstraints.HORIZONTAL, new Insets(4, 4, 4, 0), 0, 0));
		this.getContentPane().add(mainPanel,         new GridBagConstraints(0, 1, 8, 1, 1.0, 1.0
			,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(4, 0, 2, 0), 0, 0));
		this.getContentPane().add(depthLabel,         new GridBagConstraints(4, 0, 1, 1, 0.0, 0.0
			,GridBagConstraints.SOUTH, GridBagConstraints.NONE, new Insets(4, 8, 4, 4), 0, 0));
		this.getContentPane().add(depthComboBox,       new GridBagConstraints(5, 0, 1, 1, 0.0, 0.0
			,GridBagConstraints.SOUTH, GridBagConstraints.NONE, new Insets(4, 4, 4, 4), 10, 0));
		this.getContentPane().add(statusBar,         new GridBagConstraints(0, 2, 6, 1, 1.0, 0.0
			,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(0, 0, 0, 0), 0, 0));
		statusBar.add(statusLabel,      new GridBagConstraints(0, 0, 1, 1, 1.0, 0.0
			,GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(2, 2, 2, 2), 0, 0));
		this.getContentPane().add(goButton,        new GridBagConstraints(2, 0, 1, 1, 0.0, 0.0
			,GridBagConstraints.SOUTH, GridBagConstraints.NONE, new Insets(4, 4, 4, 0), 0, 0));
		this.getContentPane().add(browseButton,    new GridBagConstraints(3, 0, 1, 1, 0.0, 0.0
			,GridBagConstraints.SOUTH, GridBagConstraints.NONE, new Insets(4, 4, 4, 4), 0, 0));
		menuBar.add(editMenu);
		menuBar.add(helpMenu);
		//editMenu.add(preferencesItem);
		editMenu.add(lookAndFeelMenu);
		helpMenu.add(gettingStartedItem);
		helpMenu.addSeparator();
		helpMenu.add(aboutItem);
		lookAndFeelMenu.add(metalLAFItem);
		lookAndFeelMenu.add(platformLAFItem);
	}
}