/*
 * JDU: Disk Usage
 * 
 * Copyright (c) 2001-2005 Jason Pratt
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

package org.jasonpratt.jdu;

/**
 * @author Jason Pratt
 */
public class TableView extends javax.swing.JPanel {
	private static final long serialVersionUID = 873755198917107930L;
	protected Directory root;
	protected int depth = 4;
	protected java.awt.Color directoryColor = new java.awt.Color( 200, 230, 255 );
	protected java.awt.Color fileColor = new java.awt.Color( 255, 240, 215 );
	protected java.awt.Color tableBackgroundColor = new java.awt.Color( 96, 96, 96 );
	protected java.awt.Color tableBackgroundShadow = new java.awt.Color( 32, 32, 32 );
	protected java.awt.Color tableBackgroundHighlight = new java.awt.Color( 196, 196, 196 );
	protected java.awt.Color textColor = java.awt.Color.black;
	protected java.awt.Font textFont = new java.awt.Font( "SansSerif", java.awt.Font.PLAIN, 14 );
	protected int minFontSize = 4;
	protected java.util.HashMap areaMap = new java.util.HashMap();
	protected java.util.HashSet listeners = new java.util.HashSet();

	/////////////////////////
	// Anonymous classes
	/////////////////////////

	protected java.awt.event.MouseListener mouseListener = new java.awt.event.MouseAdapter() {
		public void mouseReleased( java.awt.event.MouseEvent ev ) {
			java.awt.Point p = ev.getPoint();
			File f = getFileForLocation( p );
			if( f instanceof Directory ) {
				Directory d = (Directory)f;
				if( TableView.this.root == d ) {
					if( d.getParent() != null ) {
						TableView.this.setRoot( d.getParent() );
					}
				} else {
					TableView.this.setRoot( d );
				}
			}
		}
	};

	//////////////////
	// Constructor
	//////////////////

	public TableView() {
		setBorder( javax.swing.BorderFactory.createBevelBorder( javax.swing.border.BevelBorder.LOWERED, tableBackgroundHighlight, tableBackgroundShadow ) );
		setBackground( tableBackgroundColor );
		addMouseListener( mouseListener );

		// tooltips
		javax.swing.ToolTipManager toolTipManager = javax.swing.ToolTipManager.sharedInstance();
		toolTipManager.setDismissDelay( 5000 );
		toolTipManager.setInitialDelay( 2000 );
		toolTipManager.setReshowDelay( 2000 );
		toolTipManager.registerComponent( this );
	}

	//////////////
	// utility
	//////////////

	public void setRoot( Directory root ) {
		Directory oldRoot = this.root;
		this.root = root;
		fireRootChanged( oldRoot, root );
		repaint();
	}

	public void setDepth( int depth ) {
		this.depth = depth;
		repaint();
	}

	public File getFileForLocation( java.awt.Point p ) {
		java.awt.Dimension size = getSize();
		if( (p.x >= 0) && (p.x < size.width) && (p.y >= 0) && (p.y < size.height) ) {
			for( java.util.Iterator iter = areaMap.keySet().iterator(); iter.hasNext(); ) { // could be done more efficiently
				java.awt.Rectangle bounds = (java.awt.Rectangle)iter.next();
				if( bounds.contains( p ) ) {
					Object o = areaMap.get( bounds );
					if( o instanceof File ) {
						return (File)o;
					} else {
						return null;
					}
				}
			}
		}

		return null;
	}

	public void addTableViewListener( org.jasonpratt.jdu.event.TableViewListener listener ) {
		listeners.add( listener );
	}

	public void removeTableViewListener( org.jasonpratt.jdu.event.TableViewListener listener ) {
		listeners.remove( listener );
	}

	protected void fireRootChanged( Directory oldRoot, Directory newRoot ) {
		for( java.util.Iterator iter = listeners.iterator(); iter.hasNext(); ) {
			((org.jasonpratt.jdu.event.TableViewListener)iter.next()).rootChanged( oldRoot, newRoot );
		}
	}

	////////////////
	// Painting
	////////////////

	public void paintComponent( java.awt.Graphics g ) {
		super.paintComponent( g );

		if( (root != null) && (depth > 0) ) {
			areaMap.clear();
			java.awt.Rectangle bounds = getBounds();
			java.awt.Insets insets = getInsets();
			bounds.x += insets.left;
			bounds.y += insets.top;
			bounds.width -= insets.left + insets.right;
			bounds.height -= insets.top + insets.bottom;

			paintDirectory( g, root, depth, bounds.x, bounds.y, bounds.width, bounds.height );
		}
	}

	protected void paintDirectory( java.awt.Graphics g, Directory dir, int depthLeft, int x, int y, int width, int height ) {
		if( (width > 1) && (height > 1) ) {
			int columnWidth = width/depthLeft;
			g.setColor( directoryColor );
			paintFile( g, dir, x, y, columnWidth, height );
			areaMap.put( new java.awt.Rectangle( x, y, columnWidth, height ), dir );

			if( depthLeft > 1 ) {
				int subX = x + columnWidth;
				int subY = y;
				int subWidth = width - columnWidth;

				for( java.util.Iterator iter = dir.getSubDirectories().iterator(); iter.hasNext(); ) {
					Directory subDir = (Directory)iter.next();
					long subHeight = ((long)height*subDir.getSize())/dir.getSize();
					if( subHeight < 2 ) {
						break;
					}
					paintDirectory( g, subDir, depthLeft - 1, subX, subY, subWidth, (int)subHeight );
					subY += subHeight;
				}

				g.setColor( fileColor );
				for( java.util.Iterator iter = dir.getFiles().iterator(); iter.hasNext(); ) {
					File subFile = (File)iter.next();
					long subHeight = ((long)height*subFile.getSize())/dir.getSize();
					if( subHeight < 2 ) {
						break;
					}
					paintFile( g, subFile, subX, subY, subWidth/(depthLeft - 1), (int)subHeight );
					areaMap.put( new java.awt.Rectangle( subX, subY, subWidth/(depthLeft - 1), (int)subHeight ), subFile );
					subY += subHeight;
				}
			}
		}
	}

	protected void paintFile( java.awt.Graphics g, File f, int x, int y, int width, int height ) {
		if( (width > 1) && (height > 1) ) {
			g.fill3DRect( x, y, width, height, true );
			if( height >= 14 ) {
				java.awt.Color c = g.getColor();
				g.setColor( textColor );
				String nameString = f.getName();
				if( nameString.trim().equals( "" ) ) {
					nameString = f.getFile().getAbsolutePath();
				}
				String sizeString = f.getSizeString();
				paintLabel( g, nameString, sizeString, x + 3, y + 1, width - 4, height - 2 );
				g.setColor( c );
			}
		}
	}

	protected void paintLabel( java.awt.Graphics g, String name, String size, int x, int y, int width, int height ) {
		boolean fontFits = false;
		boolean singleLine = false;
		java.awt.Font currentFont = textFont;
		g.setFont( currentFont );
		java.awt.FontMetrics currentMetrics = g.getFontMetrics();
		int fontSize = currentFont.getSize();
		String fontName = currentFont.getName();
		int style = currentFont.getStyle();

		// find a fit
		while( (! fontFits) && (fontSize >= minFontSize) ) {
			if( (currentMetrics.stringWidth( name ) <= width) &&
				(currentMetrics.stringWidth( size ) <= width) &&
				((2*(currentMetrics.getAscent() + currentMetrics.getDescent()) + currentMetrics.getLeading()) <= height) )
			{
				fontFits = true;
			} else if( (currentMetrics.stringWidth( name + " " + size ) <= width) &&
					   (currentMetrics.getAscent() + currentMetrics.getDescent() <= height) )
			{
				fontFits = true;
				singleLine = true;
			} else {
				fontSize--;
				currentFont = new java.awt.Font( fontName, style, fontSize );
				g.setFont( currentFont );
				currentMetrics = g.getFontMetrics();
			}
		}

		// draw the strings
		if( fontFits ) {
			if( singleLine ) {
				g.drawString( name + " " + size, x, y + currentMetrics.getAscent() );
			} else {
				g.drawString( name, x, y + currentMetrics.getAscent() );
				g.drawString( size, x, y + currentMetrics.getHeight() + currentMetrics.getLeading() + currentMetrics.getAscent() );
			}
		}
	}

	/////////////
	// ToolTips
	/////////////

	public String getToolTipText( java.awt.event.MouseEvent ev ) {
		File f = getFileForLocation( ev.getPoint() );
		if( f != null ) {
			return f.getName() + " " + f.getSizeString();
		} else {
			return null;
		}
	}
}